from enum import Enum
from typing import TypedDict

import requests
from psutil._common import bytes2human

from oihelper.auth import AuthenticationHandler
from oihelper.common import PROBLEM_DIFFICULTY, USER_AGENT


class Testcase(TypedDict):
    time: int
    memory: int
    details: str | None
    score: float
    status: str
    id: int


class Subtask(TypedDict):
    score: float
    time: int
    memory: int
    testcases: list[Testcase]
    status: str
    id: int


class RecordStatus(str, Enum):
    COMPILING = "COMPILING"
    CE = "COMPILE ERROR"
    DONE = "DONE"


class ProblemSummary(TypedDict):
    pid: str
    title: str
    difficulty: int
    color: str


class Record(TypedDict):
    score: int
    accepted: bool
    subtasks: list[Subtask]
    total_time: int
    total_memory: int
    status: RecordStatus
    msg: str
    problem: ProblemSummary
    id: str


class Submitter(object):
    def __init__(self, session: requests.Session) -> None:
        super().__init__()
        self.session = session
        self.auth_handler = AuthenticationHandler(session)

    def submit(self, pid: str, code: str) -> str:
        response = self.session.post(
            f"https://www.luogu.com.cn/fe/api/problem/submit/{pid}",
            json={"code": code},
            headers={
                "User-Agent": USER_AGENT,
                "referer": "https://www.luogu.com.cn/auth/login",
                "X-CSRF-TOKEN": self.auth_handler._get_csrf_token(),
            },
        )
        response.raise_for_status()
        return str(response.json()["rid"])

    def _get_status_type(self, status: int) -> str:
        result = "UKE"
        match status:
            case 4:
                result = "MLE"
            case 5:
                result = "TLE"
            case 6 | 14:
                result = "WA"
            case 7:
                result = "RE"
            case 12:
                result = "AC"
        return result

    def get_record(self, rid: str) -> Record:
        response = self.session.get(
            f"https://www.luogu.com.cn/record/{rid}?_contentOnly=1"
        ).json()["currentData"]["record"]
        record_details = response["detail"]

        if record_details["compileResult"] is None or response.get("score") is None:
            return {"status": RecordStatus.COMPILING}

        if not record_details["compileResult"]["success"]:
            return {
                "status": RecordStatus.CE,
                "msg": f"Compilation failed.\n\n{record_details['compileResult']['message']}",
            }

        problem_summary = ProblemSummary(
            {
                "pid": response["problem"]["pid"],
                "difficulty": PROBLEM_DIFFICULTY[response["problem"]["difficulty"]][0],
                "color": PROBLEM_DIFFICULTY[response["problem"]["difficulty"]][1],
                "title": response["problem"]["title"],
            }
        )

        result = Record(
            {
                "subtasks": [],
                "accepted": response["status"] == 12,
                "score": response["score"],
                "total_time": f"{response['time'] / 1000}s",
                "total_memory": bytes2human(response["memory"] * 1024),
                "status": RecordStatus.DONE,
                "problem": problem_summary,
                "msg": "Record pulled.",
                "id": rid,
            }
        )

        for idx, subtask in enumerate(record_details["judgeResult"]["subtasks"], 1):
            cur_result = Subtask(
                {
                    "score": subtask["score"],
                    "time": f"{subtask['time'] / 1000}s",
                    "memory": bytes2human(subtask["memory"] * 1024),
                    "testcases": [],
                    "status": self._get_status_type(subtask["status"]),
                    "id": idx,
                }
            )

            for testcase_id in range(len(subtask["testCases"])):
                try:
                    testcase = subtask["testCases"][testcase_id]
                except KeyError:
                    testcase = subtask["testCases"][str(testcase_id)]
                cur_testcase = Testcase(
                    {
                        "time": f"{testcase['time'] / 1000}s",
                        "memory": bytes2human(testcase["memory"] * 1024),
                        "details": None,
                        "score": testcase["score"],
                        "id": testcase_id + 1,
                        "status": self._get_status_type(testcase["status"]),
                    }
                )
                match testcase["status"]:
                    case 6:
                        cur_testcase["details"] = testcase["description"]
                    case 7:
                        cur_testcase[
                            "details"
                        ] = f"Received signal {testcase['signal']}."
                    case 12:
                        cur_testcase["details"] = testcase["description"]
                cur_result["testcases"].append(cur_testcase)
            result["subtasks"].append(cur_result)

        return result



import json as jsonlib
import os
import re
import sys

import click
from oihelper.judger import Judger

from .common import CONFIG_DIR, CONFIG_FILE, STATUS_COLORS, OIHelperRuntimeError, get_testcase_no, warn


def run_testcases(
    testcases: list[tuple[str, str]],
    judger: Judger,
    max_time: float,
    max_memory: int,
    msg: str = "Running testcase {id} ...",
    output: bool = True,
    testcase_type: str = "samples",
) -> None:
    summary = []
    for testcase in testcases:
        cur_summary = {
            "id": testcase[0],
            "status": "UKE",
            "details": None,
            "type": testcase_type,
        }

        if output:
            click.echo(click.style(msg.format(id=testcase[0]), bold=True, fg="blue"))

        result = judger.run_program(
            "./output", testcase[1], testcase[2], max_time, 1024 * 1024 * max_memory
        )

        status_text = click.style(
            result["status"],
            bold=True,
            fg=STATUS_COLORS.get(result["status"], STATUS_COLORS["UKE"]),
        )

        match result["status"]:
            case "AC":
                additional_info: str = (
                    f"{round(result['details']['time_cost'], 2)}s "
                    f"/ {result['details']['memory_usage']}"
                )
            case "WA":
                additional_info = result["details"]
            case "RE":
                additional_info = result["details"]
            case "TLE":
                additional_info = result["details"]
            case "MLE":
                additional_info = result["details"]
            case _:
                additional_info = click.style("No details.", fg="black")

        cur_summary["status"] = result["status"]
        cur_summary["details"] = additional_info

        if output:
            click.echo(f"  {status_text} / {additional_info}")

        summary.append(cur_summary)
    return summary


def find_local_testcases(prefix: str, testcases_dir: str):
    testcase_inputs = []
    testcase_outputs = []
    testcases = []
    testcase_input_tester = re.compile(rf"{prefix}(\d+)?.in")
    testcase_output_tester = re.compile(rf"{prefix}(\d+)?.out")
    local_input_tester = re.compile(rf"{prefix}-(\d+).in")
    local_output_tester = re.compile(rf"{prefix}-(\d+).out")

    for _root, _dirs, files in os.walk(testcases_dir):
        for filename in files:
            if testcase_input_tester.match(filename) or local_input_tester.match(
                filename
            ):
                testcase_inputs.append(filename)
            elif testcase_output_tester.match(filename) or local_output_tester.match(
                filename
            ):
                testcase_outputs.append(filename)

    testcase_path_prefix = os.path.abspath(testcases_dir)

    for _input in testcase_inputs:
        input_no = get_testcase_no(_input)
        for output in testcase_outputs:
            output_no = get_testcase_no(output)
            if input_no != output_no:
                continue
            testcases.append(
                (
                    input_no,
                    open(os.path.join(testcase_path_prefix, _input), encoding="utf-8"),
                    open(os.path.join(testcase_path_prefix, output), encoding="utf-8"),
                )
            )
            break
    return testcases


@click.command(name="run")
@click.argument("path", type=click.STRING)
@click.option("--name", "-n", default=None, help="Name of program test cases.")
@click.option(
    "--time", "-t", default=1.0, help="Maximum run time for the program in seconds."
)
@click.option(
    "--memory",
    "-m",
    default=128,
    help="Maximum memory usage for the program in megabytes.",
)
@click.option("--testcase", "-T", default="./", help="Path to the testcases directory.")
@click.option("--compiler", "-c", default="g++", help="Path to the compiler.")
@click.option(
    "--json", help="Output in JSON format or not.", is_flag=True, default=False
)
def run(
    path: str,
    name: str | None,
    time: float,
    memory: int,
    testcase: str,
    compiler: str,
    json: bool,
):
    """Run the given local file and its testcases."""
    testcase = os.path.abspath(testcase)
    path = os.path.abspath(path)

    problem_id = path.rsplit("/", 1)[-1].split(".", 1)[0]
    if name is None:
        name = problem_id

    judger = Judger(compiler)
    try:
        judger.compile_program(path, "./output")
    except RuntimeError as err:
        click.echo(f"{click.style('compilation error: ', fg='red', bold=True)}{err}")
        raise OIHelperRuntimeError

    with open(CONFIG_FILE, "r", encoding="utf-8") as config_file:
        config: dict[str, dict] = jsonlib.load(config_file)
    summary = []

    if config.get(path) is not None:
        cur = config[path]
        samples = find_local_testcases("test", f"{CONFIG_DIR}/{problem_id}")
        samples_summary = run_testcases(
            samples,
            judger,
            cur["time_limit"],
            cur["memory_limit"] / 1024,
            "Running sample {id} ...",
            not json,
        )
        summary = samples_summary
    elif not json:
        warn("No samples found in OI-Helper database.")

    testcases = find_local_testcases(name, testcase)

    if len(testcases) == 0 and not json:
        warn(f"No local testcases found in {testcase} .")
    elif not json:
        click.echo(f"Discovered {len(testcases)} local testcases in {testcase} .")

    summary.extend(
        run_testcases(
            testcases, judger, time, memory, output=not json, testcase_type="user"
        )
    )

    if json:
        click.echo(summary)



import click

from oihelper.cli.run_program import run
from oihelper.cli.parse_problem import parse
from oihelper.cli.login import login
from oihelper.cli.submit_program import submit, record


@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
def unsafe_cli():
    pass


unsafe_cli.add_command(run)
unsafe_cli.add_command(parse)
unsafe_cli.add_command(login)
unsafe_cli.add_command(submit)
unsafe_cli.add_command(record)


def cli():
    try:
        unsafe_cli()
    except Exception as err:
        click.echo(
            f"{click.style('fatal error: ', fg='red', bold=True)}{click.style(err, bold=True)}"
        )
        click.echo(
            f"{click.style('note: ', fg='blue', bold=True)}If you believe this is an internal error caused by OIHelper itself, please file an issue to OIHelper GitHub and include all error messages above."
        )
        click.echo(click.style("Exiting due to an unexpected error."))


if __name__ == "__main__":
    cli()



import os
import pathlib
import re
import click

CONFIG_DIR = os.path.expanduser(f"{pathlib.Path.home()}/oi-helper/")

CONFIG_FILE = pathlib.Path(f"{CONFIG_DIR}/config.json")

STATUS_COLORS = {
    "AC": "green",
    "WA": "red",
    "RE": "magenta",
    "TLE": "cyan",
    "MLE": "yellow",
    "UKE": "blue",
}


class OIHelperRuntimeError(Exception):
    def __init__(self, message=None) -> None:
        if message:
            self.message = message
        self.message = "An unexpected error happened during execution. There should be more expansions above."
        super().__init__(self.message)


def get_testcase_no(testcase: str) -> int:
    testcase_no = 0
    custom_testcase_tester = re.compile("\w+\d+-\d+.\w+")
    if custom_testcase_tester.match(testcase):
        return int(testcase.split("-", 1)[-1].split(".")[0])

    for char in testcase:
        if char.isdigit():
            testcase_no = testcase_no * 10 + int(char)
    return testcase_no


def abort_with_error(msg: str) -> None:
    click.echo(click.style(f"Error: {msg}", bold=True, fg="red"))
    raise click.Abort(msg)


def warn(msg: str) -> None:
    click.echo(click.style(f"Warning: {msg}", bold=True, fg="yellow"))



import json as jsonlib
import os
import pickle
import time

import click
import requests
from oihelper.submitter import Record, RecordStatus, Submitter

from .common import CONFIG_DIR, STATUS_COLORS, abort_with_error


def get_session() -> requests.Session:
    session_path = f"{CONFIG_DIR}/session.pickle"
    if not os.path.exists(session_path):
        return None
    return pickle.load(open(session_path, "rb"))


def print_record(record: Record) -> None:
    click.echo(
        click.style(f"Record {record['id']}", bold=True)
        + f" / {record['problem']['pid']} {record['problem']['title']} "
        + click.style(
            f"[{record['problem']['difficulty']}]",
            fg=record["problem"]["color"],
        )
    )
    if record["status"] == RecordStatus.COMPILING:
        click.echo(click.style("Compiling...", bold=True, fg="blue"))
        return
    if record["status"] == RecordStatus.CE:
        click.echo(click.style(record["msg"], bold=True, fg="red"))
        return
    for subtask_idx, subtask in enumerate(record["subtasks"], 1):
        click.echo(
            click.style(f"Subtask {subtask_idx}: ")
            + click.style(
                subtask["status"],
                bold=True,
                fg=STATUS_COLORS.get(subtask["status"], STATUS_COLORS["UKE"]),
            )
            + f" / {subtask['time']} / {subtask['memory']}"
        )
        for testcase in subtask["testcases"]:
            additional_details = click.style("No details.", fg="black")
            if testcase["details"] is not None:
                additional_details = testcase["details"]
            click.echo(
                "  "
                + click.style(f"#{testcase['id']}: ", bold=True, fg="blue")
                + click.style(
                    testcase["status"],
                    bold=True,
                    fg=STATUS_COLORS.get(testcase["status"], STATUS_COLORS["UKE"]),
                )
                + f" / {testcase['time']} / {testcase['memory']} / {additional_details}"
            )
    click.echo(
        "Status: "
        + click.style(
            "Accepted" if record["accepted"] else "Unaccepted",
            bold=True,
            fg=STATUS_COLORS.get("AC" if record["accepted"] else "WA"),
        )
        + f" / {record['score']}pts / {record['total_time']} / {record['total_memory']}"
    )


@click.command()
@click.argument("path", required=True)
@click.option("--pid", default=None, help="Problem id.")
@click.option(
    "--json", default=False, help="Output in JSON format or not.", is_flag=True
)
def submit(path: str, pid: str, json: bool) -> None:
    """Submit a solution to Luogu."""
    path = os.path.abspath(path)
    if pid is None:
        pid = path.rsplit("/", 1)[-1].split(".", 1)[0]
    session = get_session()
    if session is None:
        abort_with_error(
            "No login information found. Please login by using the `login` command."
        )
    submitter = Submitter(session)
    rid = submitter.submit(pid, open(path, "r").read())
    if not json:
        click.echo(
            click.style(
                f"Solution submitted with record ID {rid}.", bold=True, fg="blue"
            )
        )
        click.echo(click.style("Compiling...", bold=True))
    record = submitter.get_record(rid)
    while record["status"] == RecordStatus.COMPILING:
        record = submitter.get_record(rid)
        time.sleep(1)
    if json:
        click.echo(jsonlib.dumps(record, indent=2, ensure_ascii=False))
        return
    if record["status"] != RecordStatus.CE:
        click.echo(click.style("Compiled successfully.", bold=True))
    print_record(record)


@click.command()
@click.argument("rid", required=True)
@click.option(
    "--json", default=False, help="Output in JSON format or not.", is_flag=True
)
def record(rid: str, json: bool) -> None:
    """Get a submition record by ID."""
    session = get_session()
    if session is None:
        abort_with_error(
            "No login information found. Please login by using the `login` command."
        )
    submitter = Submitter(session)
    record = submitter.get_record(rid)
    if json:
        click.echo(jsonlib.dumps(record, indent=2, ensure_ascii=False))
        return
    print_record(record)



from html.parser import HTMLParser

import click
import requests

from .common import USER_AGENT
from oihelper.cli.common import CONFIG_DIR


class AuthenticationHandler(object):
    def __init__(self, session: requests.Session = None) -> None:
        super().__init__()
        self.session = session or requests.Session()
        self.session.headers["User-Agent"] = USER_AGENT
        self.session.headers["referer"] = "http://www.luogu.com.cn/"

    def _get_csrf_token(self) -> str:
        class HTMLCSRFTokenParser(HTMLParser):
            def handle_starttag(self, tag, attrs):
                attrs = dict(attrs)
                try:
                    if tag == "meta" and attrs["name"] == "csrf-token":
                        raise StopIteration(attrs["content"])
                except KeyError:
                    pass

        r = self.session.get("https://www.luogu.com.cn/auth/login")
        r.raise_for_status()
        try:
            HTMLCSRFTokenParser().feed(r.text)
        except StopIteration as csrf_token:
            return str(csrf_token)

    def login(self, username: str, password: str, captcha: str) -> None:
        r = self.session.post(
            "https://www.luogu.com.cn/api/auth/userPassLogin",
            headers={
                "x-csrf-token": self._get_csrf_token(),
            },
            json={
                "username": username,
                "password": password,
                "captcha": captcha,
            },
        )
        r.raise_for_status()
        return r.json()

    def show_captcha(self) -> None:
        r = self.session.get("https://www.luogu.com.cn/api/verify/captcha")
        r.raise_for_status()
        captcha_local = f"{CONFIG_DIR}/captcha.png"
        with open(captcha_local, "wb") as f:
            f.write(r.content)
        click.launch(captcha_local)

    def sync_login(self, token: str) -> None:
        self.session.post(
            "https://www.luogu.org/api/auth/syncLogin", json={"syncToken": token}
        )



import pickle
import click
import requests

from oihelper.auth import AuthenticationHandler
from .common import CONFIG_DIR, abort_with_error


@click.command()
@click.option("--username", prompt=True, help="Username of the account.")
@click.option(
    "--password", prompt=True, hide_input=True, help="Password of the account."
)
def login(username: str, password: str):
    """Login to a Luogu account."""
    auth_handler = AuthenticationHandler()
    auth_handler.show_captcha()
    captcha = click.prompt("Captcha")
    try:
        response = auth_handler.login(username, password, captcha)
    except requests.HTTPError as error:
        abort_with_error(f"Failed to login with {error}")
    auth_handler.sync_login(response["syncToken"])
    pickle.dump(auth_handler.session, open(f"{CONFIG_DIR}/session.pickle", "wb"))
    click.echo(click.style("Login success!", bold=True, fg="green"))



USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:103.0) Gecko/20100101 Firefox/103.0"

PROBLEM_DIFFICULTY = [
    ("暂未评定", "black"),
    ("入门", "red"),
    ("普及-", "yellow"),
    ("普及/提高-", "yellow"),
    ("普及+/提高", "green"),
    ("提高+/省选-", "blue"),
    ("省选/NOI-", "magenta"),
    ("NOI/NOI+/CTSC", "black-bright"),
]



import requests

from .common import USER_AGENT


class LuoguParser(object):
    def __init__(self) -> None:
        super().__init__()

    def parse_problem(self, pid: str):
        response = requests.get(
            f"https://www.luogu.com.cn/problem/{pid.upper()}?_contentOnly=1",
            headers={"User-Agent": USER_AGENT},
        ).json()["currentData"]["problem"]
        testcases: list[list[str, str]] = response["samples"]
        time_limit: int = max(response["limits"]["time"])
        memory_limit: int = max(response["limits"]["memory"])
        return {
            "testcases": testcases,
            "time_limit": time_limit,
            "memory_limit": memory_limit,
            "pid": pid.upper(),
        }



import json as jsonlib
import os
import click
from .common import CONFIG_FILE, CONFIG_DIR

from oihelper.parser import LuoguParser

DEFAULT_TEMPLATE = """\
#include <iostream>
#include <cstdio>
#include <algorithm>
#include <string>
#include <stack>
#include <queue>
using namespace std;

int main() {
    return 0;
}
"""


@click.command()
@click.argument("pid", required=True)
@click.option(
    "--json", help="Output in JSON format or not.", is_flag=True, default=False
)
def parse(pid: str, json: bool):
    """Parse the given problem's test samples and generate a blank code template."""
    pid = pid.upper()
    if not json:
        click.echo(click.style(f"Parsing problem {pid} ...", bold=True))
    try:
        with open(CONFIG_FILE, "r", encoding="utf-8") as config_file:
            config = jsonlib.load(config_file)
    except FileNotFoundError:
        if not os.path.exists(CONFIG_DIR):
            os.mkdir(CONFIG_DIR)
        config = {}
    parser = LuoguParser()
    results = parser.parse_problem(pid)
    testcase_dir = f"{CONFIG_DIR}/{pid}"
    if not os.path.exists(testcase_dir):
        os.mkdir(testcase_dir)
    for idx, testcase in enumerate(results["testcases"], 1):
        with open(
            f"{testcase_dir}/test{idx}.in", "w", encoding="utf-8"
        ) as testcase_input:
            testcase_input.write(testcase[0])
        with open(
            f"{testcase_dir}/test{idx}.out", "w", encoding="utf-8"
        ) as testcase_output:
            testcase_output.write(testcase[1])
    current_solution = os.path.abspath(f"./{pid}.cpp")
    config[current_solution] = results

    with open(CONFIG_FILE, "w", encoding="utf-8") as config_file:
        jsonlib.dump(config, config_file, indent=2)

    if not os.path.exists(current_solution):
        if not json:
            click.echo(
                click.style(
                    f"Generating a default template at {current_solution} ...",
                    bold=True,
                )
            )
        with open(current_solution, "w", encoding="utf-8") as solution:
            solution.write(DEFAULT_TEMPLATE)
    else:
        if not json:
            click.echo(
                click.style(
                    f"Warning: Source file {current_solution} already exists, "
                    f"skipping template generation.",
                    bold=True,
                    fg="yellow",
                )
            )
    if not json:
        click.echo(click.style(f"Problem {pid} parsed successfully.", bold=True))
    else:
        click.echo(jsonlib.dumps({"status": "success"}))



from collections import namedtuple
import difflib
import subprocess
import time
from threading import Thread
from typing import TextIO

from psutil._common import bytes2human  # type: ignore
from psutil import Process, NoSuchProcess  # type: ignore


pmem = namedtuple("pmem", ["rss", "vms"])


class Judger(object):
    def __init__(self, compiler_path: str = "g++") -> None:
        super().__init__()
        self.compiler_path = compiler_path.replace(" ", "\\ ")

    def _assert_equal(self, output: str, expected: str) -> dict[str, bool | str | int]:
        diffs = difflib.ndiff(output.rstrip(), expected.rstrip())

        position = -1
        for i, diff in enumerate(diffs):
            if diff[0] != " ":
                position = i
                break

        if position == -1:
            return {"passed": True, "position": -1}

        return {"passed": False, "position": position}

    def _monitor_memory_usage(
        self, process: subprocess.Popen[bytes], results: list[pmem]
    ) -> list[tuple]:
        psutil_process = Process(process.pid)
        while process.poll() is None:
            try:
                results.append(psutil_process.memory_info())
            except NoSuchProcess:
                break
            time.sleep(0.01)
        return results

    def compile_program(self, program_path: str, output_path: str = "./output") -> None:
        compile_process = subprocess.Popen(
            f'{self.compiler_path} "{program_path}" -o "{output_path}"',
            shell=True,
        )
        compile_process.wait()
        if compile_process.returncode != 0:
            raise RuntimeError(
                "Failed to compile. "
                f"There should be more error messages produced by the compiler above."
            )

    def run_program(
        self,
        exec_path: str,
        stdin: TextIO,
        stdout: TextIO,
        max_timeout: float = 1,
        max_memory: int = 1024 * 1024 * 128,
    ) -> None:
        runner_process = subprocess.Popen(
            exec_path,
            shell=True,
            stdin=stdin,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        start_time = time.time()
        results: list[pmem] = []
        memory_process = Thread(
            target=self._monitor_memory_usage, args=(runner_process, results)
        )
        memory_process.start()

        try:
            program_output = (
                runner_process.communicate(timeout=max_timeout)[0]
                .decode("utf-8")
                .rstrip("\n")
            )
            time_cost = time.time() - start_time
        except subprocess.TimeoutExpired:
            runner_process.kill()
            return {
                "status": "TLE",
                "details": f"Program ran longer than the expected {max_timeout}s.",
            }

        if runner_process.returncode != 0:
            return {
                "status": "RE",
                "details": f"Program exited with non-zero code {runner_process.returncode}.",
            }

        max_memory_usage = -1
        for result in results:
            max_memory_usage = max(result.rss, max_memory_usage)
        readable_memory_usage = bytes2human(max_memory_usage)

        if max_memory_usage > max_memory:
            return {
                "status": "MLE",
                "details": f"Exceeded maximum memory usage. "
                f"Reading {readable_memory_usage}, expected under {bytes2human(max_memory)}.",
            }

        expected_output = stdout.read().rstrip("\n")
        program_output = program_output.splitlines(True)
        for i, expected in enumerate(expected_output.splitlines(True)):
            if i >= len(program_output):
                return {
                    "status": "WA",
                    "details": f"Wrong answer on line {i} column 0, "
                    f"reading EOF, expected {expected[0]}.",
                }
            output = program_output[i]
            result = self._assert_equal(output, expected)
            if not result["passed"]:
                pos = result["position"]
                cur_output = output[pos] if pos < len(output) else "EOF"
                cur_expected = expected[pos] if pos < len(expected) else "EOF"
                return {
                    "status": "WA",
                    "details": f"Wrong answer on line {i} column {pos}, "
                    f"reading {cur_output}, expected {cur_expected}.",
                }

        return {
            "status": "AC",
            "details": {"time_cost": time_cost, "memory_usage": readable_memory_usage},
        }


if __name__ == "__main__":
    helper = Judger("g++-11")
    helper.compile_program("./test.cpp")
    helper_result = helper.run_program(
        "./output",
        open("./test.in", "r", encoding="utf-8"),
        open("./test.out", "r", encoding="utf-8"),
    )
    print(helper_result)
